@@ -15,6 +15,48 @@ This is a field that includes a list of possible valid options.
 This can be used for select and mulitple-select fields.
 Widget type is 'select'.
 
+This field type can also be used for fields that use the
+'radio_group' widget.
+
+The 'options' array can come from four different places.
+The options attribute itself, either declaratively or using a
+'build_options' method in the field, from a method in the
+form ('options_<fieldname>') or from the database.
+
+In a field declaration:
+
+   has_field 'opt_in' => ( type => 'Select', widget => 'radio_group',
+      options => [{ value => 0, label => 'No'}, { value => 1, label => 'Yes'} ] );
+
+In a custom field class:
+
+   sub build_options {
+       my $i = 0;
+       my @days = ('Sunday', 'Monday', 'Tuesda', 'Wednesday',
+           'Thursday', 'Friday', 'Saturday' ); 
+       return [
+           map {
+               {   value => $i++, label => $_ }
+           } @days
+       ];
+   }
+
+In a form:
+
+   sub options_opt_in {
+      [ { value => 0, label => 'No' }, {value => 1, label => 'Yes'} ]
+   }
+
+From a database when the name of the accessor is a relation to the
+table holding the information used to construct the select list.
+The primary key is used as the value. The other columns used are:
+
+    label_column  --  Used for the labels in the options
+    active_column --  The name of the column to be used in the query
+                      that allows the rows retrieved to be restricted 
+    sort_column   --  The name of the column used to sort the options
+
+
 =head1 METHODS
 
 =head2 options
@@ -24,16 +66,20 @@ Each has must have a label and value keys.
 
 =cut
 
-has 'options' => ( isa => 'ArrayRef[HashRef]', is => 'rw',
-                   metaclass => 'Collection::Array',
-                   auto_deref => 1,
-                   provides => {
-                      clear => 'reset_options',
-                      empty => 'has_options',
-                   },
-                   lazy => 1, 
-                   builder => 'build_options' );
-sub build_options { [] };
+has 'options' => (
+   isa        => 'ArrayRef[HashRef]',
+   is         => 'rw',
+   metaclass  => 'Collection::Array',
+   auto_deref => 1,
+   provides   => {
+      clear => 'reset_options',
+      empty => 'has_options',
+   },
+   lazy    => 1,
+   builder => 'build_options'
+);
+sub build_options { [] }
+has 'options_from' => ( isa => 'Str', is => 'rw', default => 'none' );
 
 =head2 set_options
 
@@ -41,28 +87,41 @@ Name of form method that sets options
 
 =cut
 
-has 'set_options' => ( isa => 'Str', is => 'rw',
-       default => sub {
-       my $self = shift;
-       my $name = $self->full_name;
-       $name =~ s/\./_/g;
-       return 'options_' . $name;
-    }
+sub BUILD
+{
+   my $self = shift;
+
+   $self->set_options;
+   $self->options_from('build') if $self->options && $self->has_options;
+}
+
+has 'set_options' => (
+   isa     => 'Str',
+   is      => 'rw',
+   default => sub {
+      my $self = shift;
+      my $name = $self->full_name;
+      $name =~ s/\./_/g;
+      return 'options_' . $name;
+   }
 );
+
 sub _can_options
 {
    my $self = shift;
-   return unless $self->form && 
-                 $self->set_options && 
-                 $self->form->can( $self->set_options);
+   return
+      unless $self->form &&
+         $self->set_options &&
+         $self->form->can( $self->set_options );
    return 1;
 }
+
 sub _options
 {
    my $self = shift;
    return unless $self->_can_options;
    my $meth = $self->set_options;
-   return $self->form->$meth( $self );
+   return $self->form->$meth($self);
 }
 
 =head2 multiple
@@ -80,7 +139,7 @@ at a given time.  Defaults to 0.
 
 =cut
 
-has 'size'     => ( isa => 'Int|Undef', is => 'rw', default => '0' );
+has 'size' => ( isa => 'Int|Undef', is => 'rw' );
 
 =head2 label_column
 
@@ -144,9 +203,6 @@ has 'sort_column' => ( isa => 'Str', is => 'rw' );
 
 has '+widget' => ( default => 'select' );
 
-
-
-
 =head2 select_widget
 
 If the widget is 'select' for the field then will look if the field
@@ -156,68 +212,64 @@ otherwise will return C<checkbox>.
 
 =cut
 
-sub select_widget {
-    my $field = shift;
+sub select_widget
+{
+   my $field = shift;
 
-    my $size = $field->auto_widget_size;
-    return $field->widget unless $field->widget eq 'select' && $size;
-    my $options = $field->options || [];
-    return 'select' if @$options > $size;
-    return $field->multiple ? 'checkbox' : 'radio';
+   my $size = $field->auto_widget_size;
+   return $field->widget unless $field->widget eq 'select' && $size;
+   my $options = $field->options || [];
+   return 'select' if @$options > $size;
+   return $field->multiple ? 'checkbox' : 'radio';
 }
 
 =head2 as_label
 
 Returns the option label for the option value that matches the field's current value.
 Can be helpful for displaying information about the field in a more friendly format.
-
-This does a string compare, although probably al
+This does a string compare.
 
 =cut
 
-sub as_label {
-    my $field = shift;
+sub as_label
+{
+   my $field = shift;
 
-    my $value = $field->value;
-    return unless defined $value;
+   my $value = $field->value;
+   return unless defined $value;
 
-    for ( $field->options ) {
-        return $_->{label} if $_->{value} eq $value;
-    }
-
-    return;
+   for ( $field->options )
+   {
+      return $_->{label} if $_->{value} eq $value;
+   }
+   return;
 }
 
-=head2 process 
-
-Checks that this is a multiple field if the input is an array. 
-The input value (or values if an array ref) is tested to make 
-sure they all are valid options.
-
-Returns true or false
-
-=cut
-
 sub _inner_validate_field
 {
    my ($self) = @_;
 
-   # load options because this is params validation 
+   # load options because this is params validation
    $self->_load_options;
 
-   my $input = $self->input;
-   return 1 unless defined $input;    # nothing to check
+   my $value = $self->value;
+   return 1 unless defined $value;    # nothing to check
 
-   if ( ref $input eq 'ARRAY'
-      && !( $self->can('multiple') && $self->multiple ) )
+   if ( ref $value eq 'ARRAY' &&
+      !( $self->can('multiple') && $self->multiple ) )
    {
       $self->add_error('This field does not take multiple values');
       return;
    }
+   elsif ( ref $value ne 'ARRAY' && $self->multiple )
+   {
+      $value = [$value];
+      $self->value($value);
+   }
 
    # create a lookup hash
    my %options = map { $_->{value} => 1 } $self->options;
-   for my $value ( ref $input eq 'ARRAY' ? @$input : ($input) )
+   for my $value ( ref $value eq 'ARRAY' ? @$value : ($value) )
    {
       unless ( $options{$value} )
       {
@@ -226,63 +278,47 @@ sub _inner_validate_field
       }
    }
    return 1;
-};
+}
 
 sub _init
 {
    my $self = shift;
+
+   $self->SUPER::_init;
    # load options when no input and no value (empty form )
    $self->_load_options;
 }
 
-sub _load_options_old
-{
-   my ( $self, $node, $model_stuff ) = @_;
-
-   $node ||= $self;
-   warn "HFH: load_options ", $node->name, "\n" if $self->verbose;
-   for my $field ( $node->fields ){
-       if( $field->isa( 'HTML::FormHandler::Field::Compound' ) ){
-           my $new_model_stuff = $self->compute_model_stuff( $field, $model_stuff );
-           $self->_load_options( $field, $new_model_stuff );
-       }
-       else {
-           $self->_load_field_options($field, $model_stuff);
-       }
-   }
-}
-
 sub _load_options
 {
    my $self = shift;
 
+   return if $self->options_from eq 'build';
    my @options;
-   if( $self->_can_options )
+   if ( $self->_can_options )
    {
       @options = $self->_options;
+      $self->options_from('method');
    }
-   elsif( $self->form )
+   elsif ( $self->form )
    {
-      my $full_accessor; 
+      my $full_accessor;
       $full_accessor = $self->parent->full_accessor if $self->parent;
-      @options = $self->form->lookup_options($self, $full_accessor);
+      @options = $self->form->lookup_options( $self, $full_accessor );
+      $self->options_from('model') if scalar @options;
    }
-   return unless @options; # so if there isn't an options method and no options
-                           # from a table, already set options attributes stays put
+   return unless @options;    # so if there isn't an options method and no options
+                              # from a table, already set options attributes stays put
 
-   # what's the point of this?
    @options = @{ $options[0] } if ref $options[0];
    croak "Options array must contain an even number of elements for field " . $self->name
       if @options % 2;
 
    my @opts;
    push @opts, { value => shift @options, label => shift @options } while @options;
-
    $self->options( \@opts ) if @opts;
 }
 
-
-
 =head1 AUTHORS
 
 Gerda Shank, gshank@cpan.org
