@@ -28,7 +28,7 @@ Use this module instead of C< use Moose; >
 =cut
 
 Moose::Exporter->setup_import_methods(
-   with_caller => [ 'has_field' ],
+   with_caller => [ 'has_field', 'apply' ],
    also        => 'Moose',
 );
 
@@ -48,7 +48,14 @@ sub has_field
 {
    my ( $class, $name, %options ) = @_;
 
-   $class->meta->add_to_field_list( $name, \%options );
+   $class->meta->add_to_field_list( {name => $name, %options} );
+}
+
+sub apply
+{
+   my ( $class, $arrayref ) = @_;
+
+   $class->meta->add_to_apply_list( @{$arrayref} );
 }
 
 =head1 AUTHOR
