@@ -16,7 +16,8 @@ __PACKAGE__->add_columns(
   { data_type => "VARCHAR", is_nullable => 0, size => 32 },
   "city",
   { data_type => "VARCHAR", is_nullable => 0, size => 32 },
-  "state",
+  "country_iso",
+  { data_type => "character", default_value => undef, is_nullable => 1, size => 2, },
 );
 __PACKAGE__->set_primary_key("address_id");
 
@@ -26,5 +27,10 @@ __PACKAGE__->belongs_to(
     { user_id => 'user_id' },
 );
 
+__PACKAGE__->belongs_to(
+    'country',
+    'BookDB::Schema::DB::Country',
+    { iso => 'country_iso' },
+);
 
 1;
