@@ -12,8 +12,8 @@ __PACKAGE__->add_columns(
   { data_type => "VARCHAR", is_nullable => 0, size => 16 },
   "first_name",
   { data_type => "VARCHAR", is_nullable => 0, size => 16 },
-  "country",
-  { data_type => "VARCHAR", is_nullable => 0, size => 16 },
+  "country_iso",
+  { data_type => "character", default_value => undef, is_nullable => 1, size => 2, },
   "birthdate",
   { data_type => "DATETIME", is_nullable => 0 },
 );
@@ -24,5 +24,10 @@ __PACKAGE__->set_primary_key("first_name", "last_name");
 #  { "foreign.author_id" => "self.id" },
 #);
 
+__PACKAGE__->belongs_to(
+  'country',
+  'BookDB::Schema::DB::Country',
+  { iso => 'country_iso' },
+);
 
 1;
